--[[Concept: New module to allow multiple bar setups

	Is spread across many procs - do a search for "Store" or "Shared" and it becomes obvious.]]
	
	
		
function SaveToStore(n)
	
	--Optionally just store the shared bars. Actually store all the bars, but then delete the fake store!
	if n=="Shared" then n=9999 else Store.Current=n end

	local txt
	
	if Store[n]~=nil then txt=Store[n].Text end
	Store[n]=nil
	Store.Shared=nil
	
	Store[n]={}
	Store.Shared={}
	
	if txt~=nil then Store[n].Text=txt end

	--Modified from Save()
	if Modules.popups then OpenAllPops() end --Popup module requires all popups to be open before saving. Otherwise it records the bar as being 1x1
	local index={}
	index.Main=1
	index.Shared=1
	
	for i=1,#Bar do
		--If the bar is marked for deletion, then ignore it. Otherwise - save away!
		if not(Bar[i].delete) then
			if Bar[i].Data.Shared then
				Store.Shared[index.Shared]=deepcopy(Bar[i].Data)
				Store.Shared[index.Shared].x,Store.Shared[index.Shared].y=Bar[i]:GetPosition()
				Store.Shared[index.Shared].shortcut={}
				for j=1,Store.Shared[index.Shared].sx*Store.Shared[index.Shared].sy do
					local shortcut=Bar[i].Quickslot[j]:GetShortcut()
					Store.Shared[index.Shared].shortcut[j]={}
					Store.Shared[index.Shared].shortcut[j].Data=shortcut:GetData()
					Store.Shared[index.Shared].shortcut[j].Type=shortcut:GetType()
				end
				index.Shared=index.Shared+1
			else
				Store[n][index.Main]=deepcopy(Bar[i].Data)
				Store[n][index.Main].x,Store[n][index.Main].y=Bar[i]:GetPosition()
				Store[n][index.Main].shortcut={}
				for j=1,Store[n][index.Main].sx*Store[n][index.Main].sy do
					local shortcut=Bar[i].Quickslot[j]:GetShortcut()
					Store[n][index.Main].shortcut[j]={}
					Store[n][index.Main].shortcut[j].Data=shortcut:GetData()
					Store[n][index.Main].shortcut[j].Type=shortcut:GetType()
				end
				index.Main=index.Main+1
			end
		end
	end
	if Modules.popups then CloseAllPops() end
	
	if n==9999 then Store[9999]=nil end
end

function LoadFromStore(n)
	--Save current configuration to the current slot.
	SaveToStore(Store.Current)
	Store.Current=n
	--2) Delete all bars
		for i=1,#Bar do
			Bar[i]:Delete()
			Data[i]=nil
		end
	--3) Load the Data from Store[n] & Store.Shared
		Data=nil
		local k=deepcopy(Store[n])
		   for i=1,#Store.Shared do
			  k[#k+1]=deepcopy(Store.Shared[i])
		   end
		Data=deepcopy(k)
	--4) Rebuild the bars
		Build()
	--5) Trigger each bar so it hides, if appropriate
		for i=1,#Bar do
			Bar[i]:Hide()
		end
end

function MakeStoreButton()
--Data for the Button.
	StoreButtonData=Store.StoreButtonData
	if StoreButtonData==nil then StoreButtonData={Scale=0.5,X=100,Y=100} end
	
	--Create the button.

	StoreButton=Turbine.UI.Window()
		StoreButton:SetVisible(true)
		StoreButton.DragBar = Deusdictum.UI.DragBar( StoreButton,"Bevy Bar Store")
		StoreButton.DragBar:SetZOrder(9999)
		StoreButton:SetPosition(StoreButtonData.X,StoreButtonData.Y)
		
		StoreButton:SetMouseVisible(true)

	StoreButton.Icon=Turbine.UI.Control()
		StoreButton.Icon:SetParent(StoreButton)
		StoreButton.Icon:SetVisible(true)
		StoreButton.Icon:SetSize(32,32)
		StoreButton.Icon:SetBackground("JackdawPlugins/Bevy2/BevyIcon.tga")
		StoreButton.Icon:SetStretchMode(1)
		StoreButton.Icon:SetMouseVisible(false)
		
	StoreButton.Cover=Turbine.UI.Control()
		StoreButton.Cover:SetParent(StoreButton)
		StoreButton.Cover:SetVisible(false)
		StoreButton.Cover:SetBackColor(Turbine.UI.Color(0.5,1,1,1))
		
	StoreButton.SizeChanged=function()
		if StoreButtonData==nil then Turbine.Shell.WriteLine("Nillers!") end
		StoreButton.Icon:SetSize(32*StoreButtonData.Scale,32*StoreButtonData.Scale)
		StoreButton.Cover:SetSize(32*StoreButtonData.Scale,32*StoreButtonData.Scale)
	end
		StoreButton.Icon:SetSize(32*StoreButtonData.Scale,32*StoreButtonData.Scale)
		StoreButton.Cover:SetSize(32*StoreButtonData.Scale,32*StoreButtonData.Scale)
		StoreButton:SetSize(32*StoreButtonData.Scale,32*StoreButtonData.Scale)
		StoreButton.DragBar:Refresh()

	--When UI is unlocked then make the cover visible.
		StoreButton.DragEnable=function()
			StoreButton.Cover:SetVisible(true)
		end
		
		StoreButton.DragDisable=function()
			StoreButton.Cover:SetVisible(false)
			StoreButtonData.X,StoreButtonData.Y=StoreButton:GetPosition()
		end
		
		--Cover events - resize the Button.
		StoreButton.Cover.MouseDown=function(sender,args)
			moving=true
		end
		
		StoreButton.Cover.MouseUp=function(sender,args)
			moving=false
		end
		
		StoreButton.Cover.MouseMove=function(sender,args)
				if moving then
				local x,y=StoreButton.Cover:GetMousePosition()

				StoreButtonData.Scale=math.max(x,y)/32
				if StoreButtonData.Scale<0.25 then StoreButtonData.Scale=0.25 end

				StoreButton:SetSize(32*StoreButtonData.Scale,32*StoreButtonData.Scale)
				StoreButton.DragBar:Refresh()
			end
		end
		
	StoreButton.MouseDown=function(sender,args)
			local contextMenu = Turbine.UI.ContextMenu()
			contextMenu.items = contextMenu:GetItems()
			
			for i=1,#Store do
				if Store[i].Text==nil then Store[i].Text=i end
			end
		
			if #Store>0 then
				local Load = Turbine.UI.MenuItem("Load")			
				
				contextMenu.SubMenuItems = Load:GetItems()
				
				local LoadOption={}
					for i=1,#Store do
						LoadOption[i] = Turbine.UI.MenuItem(Store[i].Text,true,Store.Current==i)
						LoadOption[i].Click = function () 
							LoadFromStore(i)
						end
						contextMenu.SubMenuItems:Add(LoadOption[i])
					end
				
				contextMenu.items:Add(Load)
			end
			
			local Save = Turbine.UI.MenuItem("Save")			
			
			contextMenu.SubMenuItems = Save:GetItems()
			
			local SaveOption={}
				for i=1,#Store do
					SaveOption[i] = Turbine.UI.MenuItem(Store[i].Text,true,Store.Current==i)
					SaveOption[i].Click = function () 
						SaveToStore(i)
					end
					contextMenu.SubMenuItems:Add(SaveOption[i])
				end
				
				local i=#Store+1
				
				SaveOption[i] = Turbine.UI.MenuItem("New",true)
				SaveOption[i].Click = function () 
					SaveToStore(#Store+1)
				end
				contextMenu.SubMenuItems:Add(SaveOption[i])
				
				i=#Store+2
				
				SaveOption[i] = Turbine.UI.MenuItem("Just save shared bars",true)
				SaveOption[i].Click = function () 
					SaveToStore("Shared")
				end
				contextMenu.SubMenuItems:Add(SaveOption[i])
			
			contextMenu.items:Add(Save)
			
			if #Store>0 then
				local Delete = Turbine.UI.MenuItem("Delete")			
				
				contextMenu.SubMenuItems = Delete:GetItems()
				
				local DeleteOption={}
					for i=1,#Store do
						DeleteOption[i] = Turbine.UI.MenuItem(Store[i].Text,true,Store.Current==i)
						DeleteOption[i].Click = function ()
							table.remove(Store, i)
						end
						contextMenu.SubMenuItems:Add(DeleteOption[i])
					end
				
				contextMenu.items:Add(Delete)
			end
			
			if #Store>0 then
				local Rename = Turbine.UI.MenuItem("Rename")			
				
				contextMenu.SubMenuItems = Rename:GetItems()
				
				local RenameOption={}
					for i=1,#Store do
						RenameOption[i] = Turbine.UI.MenuItem(Store[i].Text,true,Store.Current==i)
						RenameOption[i].Click = function () 
							RenameFromStore(i)
						end
						contextMenu.SubMenuItems:Add(RenameOption[i])
					end
				
				contextMenu.items:Add(Rename)
			end
		contextMenu:ShowMenu()
	end
end

function RenameFromStore(n)
--1) Make Window
	StoreRen={}
	
	--Make the window	
	StoreRen.Window = Turbine.UI.Lotro.Window()
	StoreRen.Window:SetPosition( Turbine.UI.Display:GetWidth()/2-150,Turbine.UI.Display:GetHeight()/2-50 )
	StoreRen.Window:SetSize(300,120)

	StoreRen.Window:SetText( "Rename Build "..n)
	StoreRen.Window:SetVisible( true )
	StoreRen.Window:SetZOrder(1000)

	StoreRen.L1=optionlabel(280,"Enter a name below;",true)
	StoreRen.L1:SetParent(StoreRen.Window)
	StoreRen.L1:SetPosition(10,40)
	
	StoreRen.Apply=Turbine.UI.Lotro.Button()
		StoreRen.Apply:SetText("Apply")
		StoreRen.Apply.MouseDown=function()
			--2) Save new name
			local name=StoreRen.Name:GetText()
			local name=string.gsub(name,"[%c]","")
			Store[n].Text=name
			StoreRen.Window:Close()
		end
		
	StoreRen.Apply:SetParent(StoreRen.Window)
	StoreRen.Apply:SetPosition(230,80)
	
	StoreRen.NameBox=Turbine.UI.Control()
		StoreRen.NameBox:SetSize(280,20)
		StoreRen.NameLabel=optionlabel(50,"Name:")
		StoreRen.Name=optiontextbox(230,20)
		StoreRen.NameLabel:SetParent(StoreRen.NameBox)
		StoreRen.Name:SetParent(StoreRen.NameBox)
		StoreRen.Name:SetLeft(50)
		
		StoreRen.Name:SetText(Store[n].Text)
		
	StoreRen.NameBox:SetParent(StoreRen.Window)
	StoreRen.NameBox:SetPosition(10,60)

	StoreRen.Window.Closed=function(sender,args)
		StoreRen=nil
	end
end

function NewBar:flagshared()
		--Bug catcher - I suspect non-existent bars.
		if self.Button==nil then return end
		--Add a faded icon to mark which bars are shared across all the stores.
		self.Button.Shared=Turbine.UI.Control()
		self.Button.Shared 	:SetPosition		( -8,-8)
		self.Button.Shared 	:SetSize			( 32,32 )
		self.Button.Shared 	:SetVisible 		( false )
		self.Button.Shared 	:SetMouseVisible 	( false )
		self.Button.Shared 	:SetOpacity			( 0.9 )
		self.Button.Shared 	:SetStretchMode		( 1 )
		self.Button.Shared 	:SetParent			( self.Button)
		self.Button.Shared	:SetBackground		("JackdawPlugins/Bevy2/BevyIcon.tga")
		self.Button.Shared 	:SetSize			( 16,16 )
	
		if self.Data.Shared then
			self.Button.Shared :SetVisible 			( true )
		end		
end

table.insert(Context,
function (self,contextMenu,contextMenuItems)
	local StoreBar = Turbine.UI.MenuItem( "Share "..self.Data.name.." across builds?",true,self.Data.Shared);
		StoreBar.Click = function( sender, args )
			if self.Data.Shared then self.Data.Shared=nil else self.Data.Shared=true end
			self:flagshared()
		end
		contextMenuItems:Add( StoreBar )
end)

table.insert(Opt.Module,
	function()
		Opt.shared=CollapseBox("Store Bar Options","f",1,true)
		Opt.ListBox:AddItem(Opt.shared)
	
			Opt.Shared=Opt.CheckBox("Share this bar across builds?",function(k,i) 
			if k then
				Bar[i].Data.Shared=true
			else
				Bar[i].Data.Shared=nil
			end
			
			Bar[i]:flagshared()
			
		end )
		Opt.shared.Content:AddItem(Opt.Shared)
		table.insert(Opt.source,"Shared")

		end)
		
if Store==nil then Store={} Store.Current=1 SaveToStore(1) end
MakeStoreButton()